#include <stdio.h>
#include <string.h>
#include "error.h"
#include "config.h"
#include "cdbslib.h"
#include "loadfile.h"
#include "reffile.h"
#include "mkload.h"
#include "cdbserror.h"

//======================================================================
//
// Name: mkload
//
// Description: Create a load (.lod) file from a FITS reference file 
//
// History:
// --------
// ??/??/?? ?     ?        original
// 05/10/11 6xxxx MSwam    port to Linux
// 06/08/11 68503 MSwam    integrate expload into mkload
//
//======================================================================
void mkload (table *catalog,      // i - already-open cdbscatalog object
             char *reffile_name   // i - name of FITS ref file to process
             )
{
    // Running this task against a LOADFILE is an ERROR
    char actual_file_type[20];
    type_reffile (reffile_name, actual_file_type);
    if (0==strcmp(actual_file_type, "load")) {
	cdbs_warn (NOLOAD, "%s", reffile_name);
        return;
    }

    // Locate the cdbscatalog.dat entry for this reffile's type
    cdbs_debug("calling find_lookup");
    int row;
    if ((row = find_lookup (catalog, reffile_name)) == ERR) {
	cdbs_warn (NOLOOK, "%s", reffile_name);
	return;
    }

    // Read types and instrument info from the row in cdbscatalog.dat
    cdbs_debug("calling get_tab_cell");
    char *instrument = get_tab_cell (catalog, "instrument", row);
    char *reftype    = get_tab_cell (catalog, "reftype", row);
    char *filetype   = get_tab_cell (catalog, "filetype", row);
    cdbs_debug("instr,reftype,type= %s %s %s",instrument,reftype,filetype);

    // Build the loadfile by creating an empty table, filling in header
    //   and body values, and assigning the comparison file
    //
    table *loadfile_table = create_table ();
    mkload_header (loadfile_table, reffile_name, instrument, 
                   reftype, actual_file_type);
    mkload_body (loadfile_table, reffile_name, instrument, 
                 reftype, actual_file_type);
    mkload_compfile (loadfile_table, instrument, reftype);

    // Run expload on the loadfile content to expand wildcards, if needed
    mkload_expload (&loadfile_table, instrument);

    // Determine the loadfile output name, and write the output loadfile
    char ldname[SZ_FNAME+1];
    mkload_name (ldname, reffile_name, SZ_FNAME);

    if (write_loadfile (ldname, loadfile_table) == BAD) {
	cdbs_warn (NOWRITE, "%s", ldname);
    } else {
	cdbs_info (WFILE, "%s", ldname);
    }
    free_table (loadfile_table);

    return;
}
